@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 // Functions that fill Tensors with constants. Implementations are in Fill.cpp.
 
 #pragma once
@@ -14,8 +15,12 @@ struct TensorIterator;
 
 namespace native {
 
-DECLARE_DISPATCH(void(*)(TensorIterator&, const c10::Scalar&), fill_stub);
+DECLARE_DISPATCH(void(*)(TensorIterator&, const c10::Scalar&), fill_stub)
 
 Tensor& fill_out(Tensor& self, const Scalar& value);
 
 }} // namespace at::native
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
