@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 #include <ATen/cuda/Exceptions.h>
 #include <ATen/cuda/CUDAContext.h>
@@ -29,3 +30,7 @@ static inline Storage pin_memory(int64_t size) {
 }
 
 } // namespace at::native
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
