@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 #include <ATen/Context.h>
@@ -15,3 +16,7 @@ C10_EXPORT bool can_use_mem_efficient_attention(sdp_params const& params, bool d
 C10_EXPORT bool can_use_cudnn_attention(sdp_params const& params, bool debug);
 
 } // namespace sdp
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
