@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 #include <array>
@@ -713,3 +714,7 @@ CAFFE_DECLARE_KNOWN_TYPE(float*, float_ptr)
 CAFFE_DECLARE_KNOWN_TYPE(at::Half*, at_Half)
 
 } // namespace caffe2
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
