@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 #include <torch/csrc/distributed/autograd/rpc_messages/autograd_metadata.h>
@@ -23,3 +24,7 @@ class TORCH_API CleanupAutogradContextReq : public rpc::RpcCommandBase {
 };
 
 } // namespace torch::distributed::autograd
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
