@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 // workaround for https://github.com/python/cpython/pull/23326
 #include <cmath>
@@ -23,3 +24,7 @@
 #if PY_MAJOR_VERSION < 3
 #error "Python 2 has reached end-of-life and is no longer supported by PyTorch."
 #endif
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
