@@ -17,6 +17,6 @@ struct alignas(1) quint8 {
 
 } // namespace c10
 
-namespace torch::headeronly {
+HIDDEN_NAMESPACE_BEGIN(torch, headeronly)
 using c10::quint8;
-} // namespace torch::headeronly
+HIDDEN_NAMESPACE_END(torch, headeronly)
