@@ -1,5 +1,3 @@
-# mypy: allow-untyped-defs
-
 """
 This module provides functionality for resuming Python execution at specific points in code,
 primarily used by PyTorch Dynamo for control flow handling and optimization. It implements
@@ -19,11 +17,16 @@ import copy
 import dataclasses
 import sys
 import types
+from collections.abc import Callable, Iterable
+from contextlib import AbstractContextManager
 from typing import Any, cast, Optional
 
 from .bytecode_transformation import (
+    add_push_null,
     bytecode_from_template,
+    create_binary_subscr,
     create_call_function,
+    create_call_function_ex,
     create_instruction,
     create_jump_absolute,
     create_load_const,
@@ -49,9 +52,11 @@ CO_ASYNC_GENERATOR = 0x0200
 
 # trace_rules.py import this constant for consistency
 TORCH_DYNAMO_RESUME_IN_PREFIX = "torch_dynamo_resume_in"
+IS_TRACING_RESUME_PROLOGUE_VARNAME = "__is_tracing_resume_prologue"
 
 
-def _initial_push_null(insts):
+# If is_resume - this codegen is for a resume function
+def _initial_push_null(insts: list[Instruction]) -> None:
     if sys.version_info >= (3, 11):
         insts.append(create_instruction("PUSH_NULL"))
         if sys.version_info < (3, 13):
@@ -59,7 +64,11 @@ def _initial_push_null(insts):
 
 
 # Generates bytecode from template and splits the code where LOAD_FAST dummy is present.
-def _bytecode_from_template_with_split(template, stack_index, varname_map=None):
+def _bytecode_from_template_with_split(
+    template: Callable[..., Any],
+    stack_index: int,
+    varname_map: Optional[dict[str, Any]] = None,
+) -> tuple[list[Instruction], list[Instruction]]:
     template_code = bytecode_from_template(template, varname_map=varname_map)
     template_code.append(create_instruction("POP_TOP"))
 
@@ -73,11 +82,12 @@ def _bytecode_from_template_with_split(template, stack_index, varname_map=None):
         (
             (i, inst)
             for i, inst in enumerate(template_code)
-            if inst.opname == "LOAD_FAST" and inst.argval == "dummy"
+            if inst.opname in ("LOAD_FAST", "LOAD_FAST_BORROW")
+            and inst.argval == "dummy"
         ),
         (None, None),
     )
-    assert dummy_idx is not None
+    assert dummy_idx is not None and dummy_inst is not None
 
     # replace LOAD_FAST dummy with first NOP marking exception area
     overwrite_instruction(dummy_inst, [create_instruction("NOP")])
@@ -89,9 +99,10 @@ def _bytecode_from_template_with_split(template, stack_index, varname_map=None):
     return template_code[: dummy_idx + 1], template_code[dummy_idx + 1 :]
 
 
-def _try_except_tf_mode_template(dummy, stack_var_name):
+def _try_except_tf_mode_template(dummy: Any, stack_var_name: Any) -> None:
     # NOTE: Make sure this name matches what is generated by symbolic_convert:import_source
     # on torch._dynamo.utils.
+    # pyrefly: ignore [unknown-name]
     global __import_torch_dot__dynamo_dot_utils
     try:
         dummy
@@ -107,7 +118,9 @@ class ReenterWith:
     stack_index: int
     target_values: Optional[tuple[Any, ...]] = None
 
-    def try_except_torch_function_mode(self, code_options, cleanup: list[Instruction]):
+    def try_except_torch_function_mode(
+        self, code_options: dict[str, Any], cleanup: list[Instruction]
+    ) -> list[Instruction]:
         """
         Codegen based off of:
         try:
@@ -129,7 +142,9 @@ class ReenterWith:
 
     # If we do not want to destroy the stack, we can do the same thing as a
     # `SETUP_WITH` block, only that we store the context manager in a local_symbol
-    def try_finally(self, code_options, cleanup: list[Instruction]):
+    def try_finally(
+        self, code_options: dict[str, Any], cleanup: list[Instruction]
+    ) -> list[Instruction]:
         """
         Codegen based off of:
         load args
@@ -160,7 +175,7 @@ class ReenterWith:
             ]
         )
 
-        def _template(ctx, dummy):
+        def _template(ctx: AbstractContextManager[Any], dummy: Any) -> None:
             ctx.__enter__()
             try:
                 dummy
@@ -173,7 +188,9 @@ class ReenterWith:
         cleanup[:] = epilogue + cleanup
         return create_ctx + setup_try_finally
 
-    def __call__(self, code_options, cleanup):
+    def __call__(
+        self, code_options: dict[str, Any], cleanup: list[Instruction]
+    ) -> tuple[list[Instruction], Optional[Instruction]]:
         """
         Codegen based off of:
         with ctx(args):
@@ -185,7 +202,9 @@ class ReenterWith:
             load_args = [create_load_const(val) for val in self.target_values]
 
         create_ctx: list[Instruction] = []
-        _initial_push_null(create_ctx)
+        # Do not push NULL in Python 3.14+ since the NULL should be on the symbolic stack.
+        if sys.version_info < (3, 14):
+            _initial_push_null(create_ctx)
         create_ctx.extend(
             [
                 *load_args,
@@ -193,7 +212,7 @@ class ReenterWith:
             ]
         )
 
-        def _template(ctx, dummy):
+        def _template(ctx: AbstractContextManager[Any], dummy: Any) -> None:
             with ctx:
                 dummy
 
@@ -206,7 +225,8 @@ class ReenterWith:
             (
                 inst
                 for inst in setup_with
-                if inst.opname == "LOAD_FAST" and inst.argval == "ctx"
+                if inst.opname in ("LOAD_FAST", "LOAD_FAST_BORROW")
+                and inst.argval == "ctx"
             ),
             None,
         )
@@ -237,11 +257,17 @@ class ResumeFunctionMetadata:
     prefix_block_target_offset_remap: list[int] = dataclasses.field(
         default_factory=list
     )
-    # map from new block target offsets to original block target offsets
-    block_target_offset_remap: Optional[dict[int, int]] = None
+    # per-offset map from new block target offsets to original block target offsets
+    block_target_offset_remap: dict[tuple[int, int], dict[int, int]] = (
+        dataclasses.field(default_factory=dict)
+    )
 
 
-def _filter_iter(l1, l2, cond):
+def _filter_iter(
+    l1: Iterable[Any],
+    l2: Iterable[Any],
+    cond: Callable[[Any, Any], bool],
+) -> list[Any]:
     """
     Two-pointer conditional filter.
     e.g. _filter_iter(insts, sorted_offsets, lambda i, o: i.offset == o)
@@ -260,7 +286,7 @@ def _filter_iter(l1, l2, cond):
     return res
 
 
-def _load_tuple_and_call(tup):
+def _load_tuple_and_call(tup: tuple[Any, ...]) -> list[Instruction]:
     insts: list[Instruction] = []
     _initial_push_null(insts)
     insts.extend(create_load_const(val) for val in tup)
@@ -273,30 +299,41 @@ class ContinueExecutionCache:
     generated_code_metadata = ExactWeakKeyDictionary()
 
     @classmethod
-    def lookup(cls, code, lineno, *key):
+    def lookup(
+        cls, code: types.CodeType, lineno: int, init_offset: int, *key: Any
+    ) -> types.CodeType:
         if code not in cls.cache:
             cls.cache[code] = {}
         key = tuple(key)
         if key not in cls.cache[code]:
-            cls.cache[code][key] = cls.generate(code, lineno, *key)
+            cls.cache[code][key] = cls.generate(code, lineno, init_offset, *key)
         return cls.cache[code][key]
 
     @classmethod
     def generate(
         cls,
-        code,
-        lineno,
-        offset: int,
+        code: types.CodeType,
+        lineno: int,
+        init_offset: int,
+        resume_offset: int,
         setup_fn_target_offsets: tuple[int, ...],  # only used in Python 3.11+
         nstack: int,
         argnames: tuple[str, ...],
         argnames_null: tuple[str, ...],
         setup_fns: tuple[ReenterWith, ...],
+        handle_inactive_ctx: bool,
         stack_ctx_vars: tuple[tuple[int, tuple[Any, ...]], ...],
         argnames_ctx_vars: tuple[tuple[str, tuple[Any, ...]], ...],
         null_idxes: tuple[int, ...],
+        # mainly used to ensure distinct code objects per stack trace,
+        # which prevents excessive recompilation of inner frames
+        nested_code_objs: tuple[types.CodeType],
+        # Are we currently graph breaking on an instruction that doesn't push
+        # its result to the stack? If so, and we are not the leaf resume, then we need to pop
+        # the result of calling the next resume function.
+        pop_nested_resume_result: bool,
     ) -> types.CodeType:
-        assert offset is not None
+        assert resume_offset is not None
         assert not (
             code.co_flags
             & (CO_GENERATOR | CO_COROUTINE | CO_ITERABLE_COROUTINE | CO_ASYNC_GENERATOR)
@@ -306,24 +343,31 @@ class ContinueExecutionCache:
             return cls.generate_based_on_original_code_object(
                 code,
                 lineno,
-                offset,
+                init_offset,
+                resume_offset,
                 setup_fn_target_offsets,
                 nstack,
                 argnames,
                 argnames_null,
                 setup_fns,
+                handle_inactive_ctx,
                 stack_ctx_vars,
                 argnames_ctx_vars,
                 null_idxes,
+                nested_code_objs,
+                pop_nested_resume_result,
             )
 
         is_py311_plus = sys.version_info >= (3, 11)
         meta = ResumeFunctionMetadata(code)
 
-        def update(instructions: list[Instruction], code_options: dict[str, Any]):
+        def update(
+            instructions: list[Instruction], code_options: dict[str, Any]
+        ) -> None:
             meta.instructions = copy.deepcopy(instructions)
 
-            args = [f"___stack{i}" for i in range(nstack)]
+            args = ["__nested_resume_fns", "__nested_frame_values"]
+            args += [f"___stack{i}" for i in range(nstack)]
             args.extend(v for v in argnames if v not in args)
             freevars = tuple(code_options["co_cellvars"] or []) + tuple(
                 code_options["co_freevars"] or []
@@ -351,16 +395,13 @@ class ContinueExecutionCache:
             code_options["co_varnames"] = tuple(
                 args
                 + [v for v in argnames_null if v not in args]
-                + [
-                    v
-                    for v in code_options["co_varnames"]
-                    if v not in args and v not in freevars
-                ]
+                + [v for v in code_options["co_varnames"] if v not in args]
+                + [IS_TRACING_RESUME_PROLOGUE_VARNAME]
             )
             code_options["co_flags"] = code_options["co_flags"] & ~(
                 CO_VARARGS | CO_VARKEYWORDS
             )
-            target = next(i for i in instructions if i.offset == offset)
+            target = next(i for i in instructions if i.offset == resume_offset)
 
             prefix = []
             if is_py311_plus:
@@ -370,6 +411,18 @@ class ContinueExecutionCache:
                     )
                 prefix.append(create_instruction("RESUME", arg=0))
 
+            # Set is_tracing_resume_prologue to prevent graph breaks.
+            # This doesn't really do anything at runtime, but dynamo will trace this
+            # and will know that we're in a resume function prologue.
+            prefix.extend(
+                [
+                    create_instruction("LOAD_CONST", argval=True),
+                    create_instruction(
+                        "STORE_FAST", argval=IS_TRACING_RESUME_PROLOGUE_VARNAME
+                    ),
+                ]
+            )
+
             cleanup: list[Instruction] = []
             hooks = {fn.stack_index: fn for fn in setup_fns}
             hook_target_offsets = {
@@ -379,16 +432,23 @@ class ContinueExecutionCache:
             offset_to_inst = {inst.offset: inst for inst in instructions}
             # map old hook targets to new targets generated by the hook
             old_hook_target_remap = {}
-            null_idxes_i = 0
+            stack_i = 0
+            null_i = 0
             stack_ctx_vars_d = dict(stack_ctx_vars)  # type: ignore[var-annotated,arg-type]
-            for i in range(nstack):
-                while (
-                    null_idxes_i < len(null_idxes)
-                    and null_idxes[null_idxes_i] == i + null_idxes_i
-                ):
+            for i in range(nstack + len(null_idxes)):
+                if null_i < len(null_idxes) and null_idxes[null_i] == i:
                     prefix.append(create_instruction("PUSH_NULL"))
-                    null_idxes_i += 1
-                prefix.append(create_instruction("LOAD_FAST", argval=f"___stack{i}"))
+                    null_i += 1
+                else:
+                    prefix.append(
+                        create_instruction("LOAD_FAST", argval=f"___stack{stack_i}")
+                    )
+                    if handle_inactive_ctx and stack_i in stack_ctx_vars_d:
+                        # NOTE: we assume that current stack var is a context manager CLASS!
+                        # Load args for context variable and construct it
+                        prefix.extend(_load_tuple_and_call(stack_ctx_vars_d[stack_i]))
+                    stack_i += 1
+
                 if i in hooks:
                     hook = hooks.pop(i)
                     hook_insts, exn_target = hook(code_options, cleanup)
@@ -398,10 +458,6 @@ class ContinueExecutionCache:
                         old_hook_target = offset_to_inst[hook_target_offset]
                         meta.prefix_block_target_offset_remap.append(hook_target_offset)
                         old_hook_target_remap[old_hook_target] = exn_target
-                if i in stack_ctx_vars_d:
-                    # NOTE: we assume that current stack var is a context manager CLASS!
-                    # Load args for context variable and construct it
-                    prefix.extend(_load_tuple_and_call(stack_ctx_vars_d[i]))
 
             if is_py311_plus:
                 # reverse the mapping since targets of later/nested contexts are inserted
@@ -414,10 +470,11 @@ class ContinueExecutionCache:
 
             # NOTE: we assume that local var is a context manager CLASS!
             # initialize inactive context vars in argnames
-            for name, vals in argnames_ctx_vars:
-                prefix.append(create_instruction("LOAD_FAST", argval=name))
-                prefix.extend(_load_tuple_and_call(vals))
-                prefix.append(create_instruction("STORE_FAST", argval=name))
+            if handle_inactive_ctx:
+                for name, vals in argnames_ctx_vars:
+                    prefix.append(create_instruction("LOAD_FAST", argval=name))
+                    prefix.extend(_load_tuple_and_call(vals))
+                    prefix.append(create_instruction("STORE_FAST", argval=name))
 
             # 3.12+: store NULL into variables that were NULL
             if argnames_null:
@@ -431,6 +488,71 @@ class ContinueExecutionCache:
                         ]
                     )
 
+            # Call nested resume function
+            if nested_code_objs:
+                prefix.extend(
+                    [
+                        # set up __nested_resume_fns[-1] call
+                        *add_push_null(
+                            [
+                                create_instruction(
+                                    "LOAD_FAST", argval="__nested_resume_fns"
+                                ),
+                                create_instruction("LOAD_CONST", argval=-1),
+                                create_binary_subscr(),
+                            ]
+                        ),
+                        # del __nested_resume_fns[-1]
+                        create_instruction("LOAD_FAST", argval="__nested_resume_fns"),
+                        create_instruction("LOAD_CONST", argval=-1),
+                        create_instruction("DELETE_SUBSCR"),
+                        # load [__nested_resume_fns, __nested_frame_values]
+                        create_instruction("LOAD_FAST", argval="__nested_resume_fns"),
+                        create_instruction("LOAD_FAST", argval="__nested_frame_values"),
+                        create_instruction("BUILD_LIST", arg=2),
+                        # load __nested_frame_values[-1]
+                        create_instruction("LOAD_FAST", argval="__nested_frame_values"),
+                        create_instruction("LOAD_CONST", argval=-1),
+                        create_binary_subscr(),
+                        # create [
+                        #     __nested_resume_fns,
+                        #     __nested_frame_values,
+                        #     *__nested_frame_values[-1],
+                        # ]
+                        create_instruction("LIST_EXTEND", arg=1),
+                        # del __nested_frame_values[-1]
+                        create_instruction("LOAD_FAST", argval="__nested_frame_values"),
+                        create_instruction("LOAD_CONST", argval=-1),
+                        create_instruction("DELETE_SUBSCR"),
+                        # delete __nested values
+                        create_instruction("DELETE_FAST", argval="__nested_resume_fns"),
+                        create_instruction(
+                            "DELETE_FAST", argval="__nested_frame_values"
+                        ),
+                        # Set is_tracing_resume_prologue back to allow graph breaks
+                        # in the nested resume
+                        create_instruction("LOAD_CONST", argval=False),
+                        create_instruction(
+                            "STORE_FAST", argval=IS_TRACING_RESUME_PROLOGUE_VARNAME
+                        ),
+                        # finish the call
+                        *create_call_function_ex(False, False),
+                    ]
+                )
+                if pop_nested_resume_result:
+                    # pop the result of calling the nested resume function
+                    prefix.append(create_instruction("POP_TOP"))
+            else:
+                # Set is_tracing_resume_prologue back to allow graph breaks after the jump
+                prefix.extend(
+                    [
+                        create_instruction("LOAD_CONST", argval=False),
+                        create_instruction(
+                            "STORE_FAST", argval=IS_TRACING_RESUME_PROLOGUE_VARNAME
+                        ),
+                    ]
+                )
+
             prefix.append(create_jump_absolute(target))
 
             # because the line number table monotonically increases from co_firstlineno
@@ -449,25 +571,26 @@ class ContinueExecutionCache:
 
             # remap original instructions' exception table entries
             if old_hook_target_remap:
+                # pyrefly: ignore [unbound-name]
                 assert is_py311_plus
                 for inst in instructions:
                     if (
                         inst.exn_tab_entry
                         and inst.exn_tab_entry.target in old_hook_target_remap
                     ):
-                        inst.exn_tab_entry.target = old_hook_target_remap[
+                        inst.exn_tab_entry.target = old_hook_target_remap[  # type: ignore[assignment]
                             inst.exn_tab_entry.target
                         ]
 
             # TODO(jansel): add dead code elimination here
             instructions[:] = prefix + instructions
 
-        new_code = transform_code_object(code, update)
+        new_code, _ = transform_code_object(code, update)
         ContinueExecutionCache.generated_code_metadata[new_code] = meta
         return new_code
 
     @staticmethod
-    def unreachable_codes(code_options) -> list[Instruction]:
+    def unreachable_codes(code_options: dict[str, Any]) -> list[Instruction]:
         """Codegen a `raise None` to make analysis work for unreachable code"""
         return [
             create_load_const(None),
@@ -476,8 +599,14 @@ class ContinueExecutionCache:
 
     @classmethod
     def generate_based_on_original_code_object(
-        cls, code, lineno, offset: int, setup_fn_target_offsets: tuple[int, ...], *args
-    ):
+        cls,
+        code: types.CodeType,
+        lineno: int,
+        init_offset: int,
+        resume_offset: int,
+        setup_fn_target_offsets: tuple[int, ...],
+        *args: Any,
+    ) -> types.CodeType:
         """
         This handles the case of generating a resume into code generated
         to resume something else.  We want to always generate starting
@@ -489,42 +618,79 @@ class ContinueExecutionCache:
         meta: ResumeFunctionMetadata = ContinueExecutionCache.generated_code_metadata[
             code
         ]
-        new_offset = None
 
-        def find_new_offset(
-            instructions: list[Instruction], code_options: dict[str, Any]
-        ):
-            nonlocal new_offset
-            (target,) = (i for i in instructions if i.offset == offset)
-            # match the functions starting at the last instruction as we have added a prefix
-            (new_target,) = (
-                i2
-                for i1, i2 in zip(reversed(instructions), reversed(meta.instructions))
-                if i1 is target
-            )
-            assert target.opcode == new_target.opcode
-            new_offset = new_target.offset
+        def find_orig_offset(cur_offset: int) -> int:
+            orig_offset = -1
 
-        transform_code_object(code, find_new_offset)
+            def find_orig_offset_transform(
+                instructions: list[Instruction], code_options: dict[str, Any]
+            ) -> None:
+                nonlocal orig_offset
+                (target,) = (i for i in instructions if i.offset == cur_offset)
+                # match the functions starting at the last instruction as we have added a prefix
+                new_target_tuple = tuple(
+                    i2
+                    for i1, i2 in zip(
+                        reversed(instructions), reversed(meta.instructions)
+                    )
+                    if i1 is target
+                )
+
+                if not new_target_tuple:
+                    # Instruction with cur_offset in instructions was not found
+                    # in the original code - orig_offset left as -1.
+                    # Caller expected to handle this case.
+                    return
+
+                assert len(new_target_tuple) == 1
+                new_target = new_target_tuple[0]
+
+                assert target.opcode == new_target.opcode
+                assert new_target.offset is not None
+                orig_offset = new_target.offset
+
+            transform_code_object(code, find_orig_offset_transform)
+            return orig_offset
+
+        orig_init_offset = find_orig_offset(init_offset)
+        # It is fine if the initial instruction is not found in the original code;
+        # this means we graph broke in the prefix, which only happens with nested graph breaks.
+        # We should not be running into ambiguous graph break issues here.
+        orig_resume_offset = find_orig_offset(resume_offset)
+        assert orig_resume_offset > -1, (
+            "resume instruction not found in original code - this is a bug."
+        )
 
         if sys.version_info >= (3, 11):
             # setup_fn_target_offsets currently contains the target offset of
             # each setup_fn, based on `code`. When we codegen the resume function
             # based on the original code object, `meta.code`, the offsets in
             # setup_fn_target_offsets must be based on `meta.code` instead.
-            if not meta.block_target_offset_remap:
-                block_target_offset_remap = meta.block_target_offset_remap = {}
+            offset_key = (orig_init_offset, orig_resume_offset)
+            # NOTE: we key by offset_key since the same resume function may graph
+            # break in multiple places and we need different block_target_offset_remap's
+            # for each graph break location. Keying by orig_resume_offset may not be enough
+            # if 2 graph breaks on different initial offsets resume on the same instruction
+            # (although this is rare and not tested anywhere).
+            if offset_key not in meta.block_target_offset_remap:
+                block_target_offset_remap = meta.block_target_offset_remap[
+                    offset_key
+                ] = {}
 
                 def remap_block_offsets(
                     instructions: list[Instruction], code_options: dict[str, Any]
-                ):
+                ) -> None:
                     # NOTE: each prefix block generates exactly one PUSH_EXC_INFO,
                     # so we can tell which block a prefix PUSH_EXC_INFO belongs to,
-                    # by counting. Then we can use meta.prefix_block-target_offset_remap
+                    # by counting. Then we can use meta.prefix_block_target_offset_remap
                     # to determine where in the original code the PUSH_EXC_INFO offset
                     # replaced.
                     prefix_blocks: list[Instruction] = []
                     for inst in instructions:
+                        # NOTE meta.prefix_block_target_offset_remap is based off of how we codegen'd
+                        # context managers at the prefix/prologue of the resume function. It is the same for
+                        # every graph break in the same resume function, so we do not need to recompute
+                        # for each graph break (unlike for meta.block_target_offset_remap)
                         if len(prefix_blocks) == len(
                             meta.prefix_block_target_offset_remap
                         ):
@@ -532,37 +698,49 @@ class ContinueExecutionCache:
                         if inst.opname == "PUSH_EXC_INFO":
                             prefix_blocks.append(inst)
 
-                    # offsets into prefix
+                    # remap block target offsets for blocks generated in the resume prefix
                     for inst, o in zip(
                         prefix_blocks, meta.prefix_block_target_offset_remap
                     ):
                         block_target_offset_remap[cast(int, inst.offset)] = o
 
-                    # old bytecode targets are after the prefix PUSH_EXC_INFO's
-                    old_start_offset = (
+                    # current bytecode targets are after the prefix PUSH_EXC_INFO's
+                    cur_start_offset = (
                         cast(int, prefix_blocks[-1].offset) if prefix_blocks else -1
                     )
-                    # offsets into old bytecode
-                    old_inst_offsets = sorted(
-                        n for n in setup_fn_target_offsets if n > old_start_offset
+                    # get the remaining block target offsets of the current bytecode
+                    cur_inst_offsets = sorted(
+                        n for n in setup_fn_target_offsets if n > cur_start_offset
                     )
                     targets = _filter_iter(
-                        instructions, old_inst_offsets, lambda inst, o: inst.offset == o
+                        instructions, cur_inst_offsets, lambda inst, o: inst.offset == o
                     )
-                    new_targets = _filter_iter(
-                        zip(reversed(instructions), reversed(meta.instructions)),
-                        targets,
-                        lambda v1, v2: v1[0] is v2,
+                    # The original code and resume code should have matching suffixes.
+                    # Match the post-prefix block target offsets of the current resume code
+                    # and the original code.
+                    orig_targets = reversed(
+                        _filter_iter(
+                            zip(reversed(instructions), reversed(meta.instructions)),
+                            reversed(targets),
+                            lambda v1, v2: v1[0] is v2,
+                        )
                     )
-                    for new, old in zip(new_targets, targets):
-                        block_target_offset_remap[old.offset] = new[1].offset
+                    for orig, cur in zip(orig_targets, targets):
+                        block_target_offset_remap[cur.offset] = orig[1].offset
 
                 transform_code_object(code, remap_block_offsets)
 
-            # if offset is not in setup_fn_target_offsets, it is an error
+            # if offset_key or offset is not in setup_fn_target_offsets, it is an error
+            # that needs to be fixed
             setup_fn_target_offsets = tuple(
-                meta.block_target_offset_remap[n] for n in setup_fn_target_offsets
+                meta.block_target_offset_remap[offset_key][n]
+                for n in setup_fn_target_offsets
             )
         return ContinueExecutionCache.lookup(
-            meta.code, lineno, new_offset, setup_fn_target_offsets, *args
+            meta.code,
+            lineno,
+            orig_init_offset,
+            orig_resume_offset,
+            setup_fn_target_offsets,
+            *args,
         )
