@@ -7,7 +7,6 @@ from .base import TemplateConfigHeuristics
 
 
 if TYPE_CHECKING:
-    from ..ir import Layout
     from ..kernel_inputs import KernelInputs
 
 
@@ -19,10 +18,9 @@ class AddMMConfigMixin(TemplateConfigHeuristics):
     def get_extra_kwargs(
         self,
         kernel_inputs: KernelInputs,
-        layout: Layout,
         op_name: str,
     ) -> dict[str, Any]:
-        kwargs = super().get_extra_kwargs(kernel_inputs, layout, op_name)
+        kwargs = super().get_extra_kwargs(kernel_inputs, op_name)
         assert op_name in [
             "addmm",
             "baddbmm",
@@ -31,7 +29,9 @@ class AddMMConfigMixin(TemplateConfigHeuristics):
         beta = kernel_inputs.get_scalar("beta")
         return {
             **kwargs,
-            "epilogue_fn": addmm_epilogue(layout.dtype, alpha, beta),
-            "epilogue_fn_hash": str(["addmm_epilogue", layout.dtype, alpha, beta]),
+            "epilogue_fn": addmm_epilogue(kernel_inputs.out_dtype(), alpha, beta),
+            "epilogue_fn_hash": str(
+                ["addmm_epilogue", kernel_inputs.out_dtype(), alpha, beta]
+            ),
             "prefix_args": 1,
         }
