@@ -2,7 +2,8 @@
 import collections
 import copy
 import operator
-from typing import Any, Callable, Optional
+from collections.abc import Callable
+from typing import Any
 
 import torch
 import torch.fx
@@ -93,8 +94,11 @@ class OutputProp:
                 )
 
             if isinstance(result, torch.Tensor):  # type: ignore[possibly-undefined]
+                # pyrefly: ignore [unbound-name]
                 node.traced_result = result
 
+            # pyrefly: ignore [unsupported-operation]
+            # pyrefly: ignore [unbound-name]
             env[node.name] = result
 
         return None
@@ -141,9 +145,11 @@ def _get_dedup_subgraphs(matches: dict[str, _MatchResult]) -> dict[str, list[Nod
                 seen_nodes.add(node_or_tuple)
 
             else:
-                assert isinstance(node_or_tuple, tuple)
+                if not isinstance(node_or_tuple, tuple):
+                    raise AssertionError(f"Expected tuple, got {type(node_or_tuple)}")
                 for node in node_or_tuple:
-                    assert isinstance(node, Node)
+                    if not isinstance(node, Node):
+                        raise AssertionError(f"Expected Node, got {type(node)}")
                     if node in seen_nodes:
                         was_seen = True
                     seen_nodes.add(node)
@@ -157,7 +163,10 @@ def _get_dedup_subgraphs(matches: dict[str, _MatchResult]) -> dict[str, list[Nod
         if len(cur_match[1]) == 1:
             list_of_nodes = cur_match[1]
         else:
-            assert len(cur_match[1]) == 2
+            if len(cur_match[1]) != 2:
+                raise ValueError(
+                    f"Expected cur_match[1] to have length 2, got {len(cur_match[1])}"
+                )
             # either (a, b), or ((a, b), c) or (c, (a, b))
             # cannot make any assumptions on order, not clear what the
             # _find_matches function is doing to populate this
@@ -178,13 +187,12 @@ def _get_dedup_subgraphs(matches: dict[str, _MatchResult]) -> dict[str, list[Nod
                         last_node = n
                     else:
                         mid_node = n
-                assert (
-                    first_node is not None
-                    and mid_node is not None
-                    and last_node is not None
-                )
-                assert mid_node.args[0] is first_node
-                assert last_node.args[0] is mid_node
+                if first_node is None or mid_node is None or last_node is None:
+                    raise AssertionError("Expected all nodes to be non-None")
+                if mid_node.args[0] is not first_node:
+                    raise AssertionError("Expected mid_node.args[0] to be first_node")
+                if last_node.args[0] is not mid_node:
+                    raise AssertionError("Expected last_node.args[0] to be mid_node")
                 return [last_node, mid_node, first_node]
 
             if isinstance(cur_match[1][0], Node) and isinstance(cur_match[1][1], Node):
@@ -217,7 +225,7 @@ def _get_logger_for_subgraph(
     subgraph_candidate_idx: int,
     qconfig_str: str,
     logger_cls: Callable,
-    fqn: Optional[str],
+    fqn: str | None,
 ) -> torch.nn.Module:
     """
     Given a model and a linear subgraph starting from `first_node` and
@@ -374,7 +382,10 @@ def create_submodule_from_subgraph(
             # the current implementation is simplistic and cannot handle
             # ops with two or more arguments which need to be passed from
             # the previous op, so we assert them out
-            assert cur_node_orig.target not in BINARY_FUNCTIONS
+            if cur_node_orig.target in BINARY_FUNCTIONS:
+                raise AssertionError(
+                    f"Unexpected binary function target: {cur_node_orig.target}"
+                )
 
             # at this point in the code, cur_node_copy is pointing to the copy
             # of the previous node
@@ -393,8 +404,10 @@ def create_submodule_from_subgraph(
                         cur_name_idx += 1
                         setattr(gm, mod_name, new_arg)
                         new_arg_placeholder = gm.placeholder(mod_name)  # type: ignore[operator]
+                        # pyrefly: ignore [missing-attribute]
                         cur_args_copy.append(new_arg_placeholder)
                     elif isinstance(arg, (float, int, torch.dtype)):
+                        # pyrefly: ignore [missing-attribute]
                         cur_args_copy.append(arg)
                     else:
                         raise AssertionError(f"arg of type {type(arg)} not handled yet")
@@ -430,9 +443,10 @@ def create_submodule_from_subgraph(
             break
 
         # go to next node
-        assert len(cur_node_orig.users.keys()) == 1, (
-            f"{cur_node_orig} has more than 1 users, not supported yet"
-        )
+        if len(cur_node_orig.users.keys()) != 1:
+            raise AssertionError(
+                f"{cur_node_orig} has more than 1 users, not supported yet"
+            )
         cur_node_orig = next(iter(cur_node_orig.users.keys()))
         cur_iteration += 1
         if cur_iteration > iteration_limit:
@@ -451,12 +465,12 @@ def create_one_transformed_and_logged_copy_of_subgraph(
     subgraph_candidate_idx: int,
     first_node: Node,
     last_node: Node,
-    fqn: Optional[str],
+    fqn: str | None,
     list_of_node_name_to_qconfig: list[dict[str, QConfigAny]],
     example_inputs: Any,
-    last_added_shadow_node_list: list[Optional[Node]],
-    custom_prepare_fn: Optional[Callable] = None,
-    custom_prepare_kwargs: Optional[dict[str, Any]] = None,
+    last_added_shadow_node_list: list[Node | None],
+    custom_prepare_fn: Callable | None = None,
+    custom_prepare_kwargs: dict[str, Any] | None = None,
 ) -> None:
     """
     Given a subgraph in `mt` and a subgraph candidate idx, inserts the
@@ -489,7 +503,8 @@ def create_one_transformed_and_logged_copy_of_subgraph(
         )
 
         attr_name = _get_attr_name(subgraph_idx, subgraph_candidate_idx)
-        assert not hasattr(mt, attr_name)
+        if hasattr(mt, attr_name):
+            raise AssertionError(f"Unexpected attribute '{attr_name}' found in {mt}")
         setattr(mt, attr_name, logger_mod_orig)
         with mt.graph.inserting_after(last_node):
             new_node = mt.graph.call_module(attr_name, args=(last_node,), kwargs={})
@@ -532,9 +547,10 @@ def create_one_transformed_and_logged_copy_of_subgraph(
                 "prepare_custom_config",
                 "qconfig_mapping",
             ]:
-                assert kwarg_name not in custom_prepare_kwargs, (
-                    f"cannot specify {kwarg_name} in custom_prepare_kwargs"
-                )
+                if kwarg_name in custom_prepare_kwargs:
+                    raise AssertionError(
+                        f"cannot specify {kwarg_name} in custom_prepare_kwargs"
+                    )
             prepare_kwargs: dict[str, Any] = {
                 "example_inputs": example_inputs,
                 "qconfig_mapping": qconfig_mapping,
@@ -546,7 +562,8 @@ def create_one_transformed_and_logged_copy_of_subgraph(
 
         # attach the wrapper to the model
         attr_name = _get_attr_wrapper_name(subgraph_idx, subgraph_candidate_idx)
-        assert not hasattr(mt, attr_name)
+        if hasattr(mt, attr_name):
+            raise AssertionError(f"Unexpected attribute '{attr_name}' found in {mt}")
         setattr(mt, attr_name, orig_mod_copy_wrapped)
 
         # add a call to the wrapper module from the parent graph
@@ -595,7 +612,8 @@ def create_one_transformed_and_logged_copy_of_subgraph(
         )
 
         attr_name = _get_attr_name(subgraph_idx, subgraph_candidate_idx)
-        assert not hasattr(mt, attr_name)
+        if hasattr(mt, attr_name):
+            raise AssertionError(f"Unexpected attribute '{attr_name}' found in {mt}")
         setattr(mt, attr_name, logger_mod_orig)
         with mt.graph.inserting_after(new_node):
             logger = mt.graph.call_module(
@@ -613,8 +631,8 @@ def create_n_transformed_and_logged_copies_of_subgraph(
     nodes_in_this_subgraph: list[Any],
     qconfig_mappings: list[QConfigMapping],
     list_of_node_name_to_qconfig: list[dict[str, QConfigAny]],
-    custom_prepare_fn: Optional[Callable] = None,
-    custom_prepare_kwargs: Optional[dict[str, Any]] = None,
+    custom_prepare_fn: Callable | None = None,
+    custom_prepare_kwargs: dict[str, Any] | None = None,
 ) -> None:
     """
     Given a model `mt` and a subgraph_idx, creates the needed copies
@@ -691,7 +709,7 @@ def create_n_transformed_and_logged_copies_of_subgraph(
     # order but the eventual results will be in reverse order.
     # So, we keep track of the last shadow logger we added and
     # always insert after it.
-    last_added_shadow_node_list: list[Optional[Node]] = [None]
+    last_added_shadow_node_list: list[Node | None] = [None]
     for subgraph_candidate_idx in range(len(qconfig_mappings) + 1):
         create_one_transformed_and_logged_copy_of_subgraph(
             mt,
@@ -801,6 +819,7 @@ def create_add_loggers_graph(
                 model,
                 cur_subgraph_idx,
                 match_name,
+                # pyrefly: ignore [bad-argument-type]
                 maybe_subgraph,
                 [qconfig_mapping],
                 [node_name_to_qconfig],
@@ -818,7 +837,8 @@ def create_add_loggers_graph(
                 ):
                     new_shadow_mod = maybe_shadow_mod
                     break
-            assert new_shadow_mod is not None
+            if new_shadow_mod is None:
+                raise AssertionError("Expected new_shadow_mod to be non-None")
             orig_first_node_to_shadow_in_node[first_node] = new_shadow_mod
             orig_first_node_to_shadow_out_node[first_node] = new_shadow_mod
 
@@ -844,7 +864,10 @@ def create_add_loggers_graph(
                 fqn,
             )
             attr_name = _get_attr_name(cur_subgraph_idx, subgraph_candidate_idx)
-            assert not hasattr(model, attr_name)
+            if hasattr(model, attr_name):
+                raise AssertionError(
+                    f"Unexpected attribute '{attr_name}' found in {model}"
+                )
             setattr(model, attr_name, logger_mod_orig)
             insertion_point = last_node
             with model.graph.inserting_after(insertion_point):
@@ -857,13 +880,14 @@ def create_add_loggers_graph(
             cur_node_orig = first_node
             cur_node_copy = None
             first_node_copy = None
+            # pyrefly: ignore [bad-assignment]
             while cur_node_orig in subgraph_to_use:
                 # TODO(future PR): make this support all possible args/kwargs
                 if cur_node_orig is first_node:
                     new_args = cur_node_orig.args
                     new_kwargs = cur_node_orig.kwargs
                 else:
-                    first_arg_for_copy: Optional[Node] = cur_node_copy
+                    first_arg_for_copy: Node | None = cur_node_copy
                     new_args = (first_arg_for_copy, *cur_node_orig.args[1:])
                     new_kwargs = cur_node_orig.kwargs
                 # make a copy of cur_node_orig
@@ -880,9 +904,15 @@ def create_add_loggers_graph(
                 # since now only linear subgraphs are supported, all nodes
                 # except the last one must have only one user
                 if cur_node_orig != last_node:
-                    assert len(cur_node_orig.users.keys()) == 1
+                    if len(cur_node_orig.users.keys()) != 1:
+                        raise AssertionError(
+                            f"Expected exactly 1, but got {len(cur_node_orig.users)}"
+                        )
                 cur_node_orig = next(iter(cur_node_orig.users.keys()))
-                assert not cur_node_orig.name.startswith(SHADOW_NODE_NAME_PREFIX)
+                if cur_node_orig.name.startswith(SHADOW_NODE_NAME_PREFIX):
+                    raise AssertionError(
+                        "cur_node_orig should not start with SHADOW_NODE_NAME_PREFIX"
+                    )
                 insertion_point = cur_node_copy
 
             # add a comparison logger after last_node's copy
@@ -898,7 +928,10 @@ def create_add_loggers_graph(
                 fqn,
             )
             attr_name = _get_attr_name(cur_subgraph_idx, subgraph_candidate_idx)
-            assert not hasattr(model, attr_name)
+            if hasattr(model, attr_name):
+                raise AssertionError(
+                    f"Unexpected attribute '{attr_name}' found in {model}"
+                )
             setattr(model, attr_name, logger_mod_orig)
             with model.graph.inserting_after(insertion_point):
                 logger = model.graph.call_module(
@@ -972,7 +1005,8 @@ def create_add_loggers_graph(
             return prev_shadow_output
 
         cur_shadow_input = orig_first_node_to_shadow_in_node[first_node]
-        assert cur_shadow_input is not None
+        if cur_shadow_input is None:
+            raise AssertionError("Expected cur_shadow_input to be non-None")
         cur_shadow_input.args = tree_map(
             maybe_remap_node_to_shadow, cur_shadow_input.args
         )
@@ -1012,16 +1046,20 @@ def _get_weight_info_from_shadow_wrapper(shadow_wrapper: torch.nn.Module):
         #  we have `w2_0`, and are navigating this subgraph
         #  to get `_input_scale_1` and `_input_zero_point_1`
 
-        assert len(shadow_n.users) == 1
+        if len(shadow_n.users) != 1:
+            raise AssertionError(f"Expected exactly 1, got {len(shadow_n.users)}")
         quant_node = next(iter(shadow_n.users.keys()))
         new_args: Any = None
-        if quant_node.target == torch.quantize_per_channel:
+        if quant_node.target is torch.quantize_per_channel:
             _weight, scale_node, zp_node, axis, dtype = quant_node.args
             scale_val = getattr_from_fqn(shadow_wrapper, scale_node.target)
             zp_val = getattr_from_fqn(shadow_wrapper, zp_node.target)
             new_args = (scale_val, zp_val, axis, dtype)
         else:
-            assert quant_node.target == torch.quantize_per_tensor
+            if quant_node.target != torch.quantize_per_tensor:
+                raise AssertionError(
+                    f"Expected torch.quantize_per_tensor, but got {quant_node.target}"
+                )
             _weight, scale_node, zp_node, dtype = quant_node.args
             scale_val = getattr_from_fqn(shadow_wrapper, scale_node.target)
             zp_val = getattr_from_fqn(shadow_wrapper, zp_node.target)
