@@ -3,12 +3,14 @@ from __future__ import annotations
 
 from dataclasses import dataclass
 from enum import Enum
-from typing import Any, Callable, Optional, TYPE_CHECKING, Union
+from typing import Any, TYPE_CHECKING
 
 import torch
 
 
 if TYPE_CHECKING:
+    from collections.abc import Callable
+
     from torch.ao.quantization.utils import Pattern
 
 
@@ -100,13 +102,13 @@ class DTypeWithConstraints:
       the quantization parameters don't match, then the QConfig will be ignored.
     """
 
-    dtype: Optional[torch.dtype] = None
-    quant_min_lower_bound: Union[int, float, None] = None
-    quant_max_upper_bound: Union[int, float, None] = None
-    scale_min_lower_bound: Union[int, float, None] = None
-    scale_max_upper_bound: Union[int, float, None] = None
-    scale_exact_match: Optional[float] = None
-    zero_point_exact_match: Optional[int] = None
+    dtype: torch.dtype | None = None
+    quant_min_lower_bound: int | float | None = None
+    quant_max_upper_bound: int | float | None = None
+    scale_min_lower_bound: int | float | None = None
+    scale_max_upper_bound: int | float | None = None
+    scale_exact_match: float | None = None
+    zero_point_exact_match: int | None = None
 
 
 @dataclass
@@ -183,16 +185,16 @@ scale_min_lower_bound=None, scale_max_upper_bound=None)
     input_dtype_with_constraints: DTypeWithConstraints
     output_dtype_with_constraints: DTypeWithConstraints
     weight_dtype_with_constraints: DTypeWithConstraints
-    bias_dtype: Optional[torch.dtype]
-    is_dynamic: Optional[bool]
+    bias_dtype: torch.dtype | None
+    is_dynamic: bool | None
 
     def __init__(
         self,
-        input_dtype: Union[torch.dtype, DTypeWithConstraints, None] = None,
-        output_dtype: Union[torch.dtype, DTypeWithConstraints, None] = None,
-        weight_dtype: Union[torch.dtype, DTypeWithConstraints, None] = None,
-        bias_dtype: Optional[torch.dtype] = None,
-        is_dynamic: Optional[bool] = None,
+        input_dtype: torch.dtype | DTypeWithConstraints | None = None,
+        output_dtype: torch.dtype | DTypeWithConstraints | None = None,
+        weight_dtype: torch.dtype | DTypeWithConstraints | None = None,
+        bias_dtype: torch.dtype | None = None,
+        is_dynamic: bool | None = None,
     ):
         if isinstance(input_dtype, DTypeWithConstraints):
             self.input_dtype_with_constraints = input_dtype
@@ -217,15 +219,15 @@ scale_min_lower_bound=None, scale_max_upper_bound=None)
         self.is_dynamic = is_dynamic
 
     @property
-    def input_dtype(self) -> Optional[torch.dtype]:
+    def input_dtype(self) -> torch.dtype | None:
         return self.input_dtype_with_constraints.dtype
 
     @property
-    def output_dtype(self) -> Optional[torch.dtype]:
+    def output_dtype(self) -> torch.dtype | None:
         return self.output_dtype_with_constraints.dtype
 
     @property
-    def weight_dtype(self) -> Optional[torch.dtype]:
+    def weight_dtype(self) -> torch.dtype | None:
         return self.weight_dtype_with_constraints.dtype
 
     @classmethod
@@ -238,29 +240,29 @@ scale_min_lower_bound=None, scale_max_upper_bound=None)
             "bias_type": torch.dtype
             "is_dynamic": bool
         """
-        input_dtype = dtype_config_dict.get(INPUT_DTYPE_DICT_KEY, None)
+        input_dtype = dtype_config_dict.get(INPUT_DTYPE_DICT_KEY)
         if input_dtype is not None and not isinstance(
             input_dtype, (torch.dtype, DTypeWithConstraints)
         ):
             raise ValueError(
                 "Expected input_dtype to be a torch.dtype or DTypeWithConstraints"
             )
-        output_dtype = dtype_config_dict.get(OUTPUT_DTYPE_DICT_KEY, None)
+        output_dtype = dtype_config_dict.get(OUTPUT_DTYPE_DICT_KEY)
         if output_dtype is not None and not isinstance(
             output_dtype, (torch.dtype, DTypeWithConstraints)
         ):
             raise ValueError(
                 "Expected output_dtype to be a torch.dtype or DTypeWithConstraints"
             )
-        weight_dtype = dtype_config_dict.get(WEIGHT_DTYPE_DICT_KEY, None)
+        weight_dtype = dtype_config_dict.get(WEIGHT_DTYPE_DICT_KEY)
         if weight_dtype is not None and not isinstance(
             weight_dtype, (torch.dtype, DTypeWithConstraints)
         ):
             raise ValueError(
                 "Expected weight_dtype to be a torch.dtype or DTypeWithConstraints"
             )
-        bias_dtype = dtype_config_dict.get(BIAS_DTYPE_DICT_KEY, None)
-        is_dynamic = dtype_config_dict.get(IS_DYNAMIC_DICT_KEY, None)
+        bias_dtype = dtype_config_dict.get(BIAS_DTYPE_DICT_KEY)
+        is_dynamic = dtype_config_dict.get(IS_DYNAMIC_DICT_KEY)
         return cls(input_dtype, output_dtype, weight_dtype, bias_dtype, is_dynamic)
 
     def to_dict(self) -> dict[str, Any]:
@@ -440,22 +442,22 @@ class BackendPatternConfig:
     For a detailed example usage, see :class:`~torch.ao.quantization.backend_config.BackendConfig`.
     """
 
-    def __init__(self, pattern: Optional[Pattern] = None):
-        self.pattern: Optional[Pattern] = pattern
+    def __init__(self, pattern: Pattern | None = None):
+        self.pattern: Pattern | None = pattern
         self.observation_type = ObservationType.OUTPUT_USE_DIFFERENT_OBSERVER_AS_INPUT
         self.dtype_configs: list[DTypeConfig] = []
-        self.root_module: Optional[type[torch.nn.Module]] = None
-        self.qat_module: Optional[type[torch.nn.Module]] = None
-        self.reference_quantized_module: Optional[type[torch.nn.Module]] = None
-        self.fused_module: Optional[type[torch.nn.Module]] = None
-        self.fuser_method: Optional[Callable] = None
+        self.root_module: type[torch.nn.Module] | None = None
+        self.qat_module: type[torch.nn.Module] | None = None
+        self.reference_quantized_module: type[torch.nn.Module] | None = None
+        self.fused_module: type[torch.nn.Module] | None = None
+        self.fuser_method: Callable | None = None
 
         # Temporary/internal configs
-        self._root_node_getter: Optional[Callable] = None
-        self._extra_inputs_getter: Optional[Callable] = None
+        self._root_node_getter: Callable | None = None
+        self._extra_inputs_getter: Callable | None = None
         self._num_tensor_args_to_observation_type: dict[int, ObservationType] = {}
         self._input_type_to_index: dict[str, int] = {}
-        self._pattern_complex_format: Optional[Pattern] = None
+        self._pattern_complex_format: Pattern | None = None
 
     def __repr__(self):
         dict_nonempty = {
@@ -671,23 +673,23 @@ class BackendPatternConfig:
         for d in backend_pattern_config_dict.get(DTYPE_CONFIGS_DICT_KEY, []):
             conf.add_dtype_config(_get_dtype_config(d))
         conf.set_root_module(
-            backend_pattern_config_dict.get(ROOT_MODULE_DICT_KEY, None)  # type: ignore[arg-type]
+            backend_pattern_config_dict.get(ROOT_MODULE_DICT_KEY)  # type: ignore[arg-type]
         )
-        conf.set_qat_module(backend_pattern_config_dict.get(QAT_MODULE_DICT_KEY, None))  # type: ignore[arg-type]
+        conf.set_qat_module(backend_pattern_config_dict.get(QAT_MODULE_DICT_KEY))  # type: ignore[arg-type]
         conf.set_reference_quantized_module(
-            backend_pattern_config_dict.get(REFERENCE_QUANTIZED_MODULE_DICT_KEY, None)  # type: ignore[arg-type]
+            backend_pattern_config_dict.get(REFERENCE_QUANTIZED_MODULE_DICT_KEY)  # type: ignore[arg-type]
         )
         conf.set_fused_module(
-            backend_pattern_config_dict.get(FUSED_MODULE_DICT_KEY, None)  # type: ignore[arg-type]
+            backend_pattern_config_dict.get(FUSED_MODULE_DICT_KEY)  # type: ignore[arg-type]
         )
         conf.set_fuser_method(
-            backend_pattern_config_dict.get(FUSER_METHOD_DICT_KEY, None)  # type: ignore[arg-type]
+            backend_pattern_config_dict.get(FUSER_METHOD_DICT_KEY)  # type: ignore[arg-type]
         )
         conf._set_root_node_getter(
-            backend_pattern_config_dict.get(ROOT_NODE_GETTER_DICT_KEY, None)  # type: ignore[arg-type]
+            backend_pattern_config_dict.get(ROOT_NODE_GETTER_DICT_KEY)  # type: ignore[arg-type]
         )
         conf._set_extra_inputs_getter(
-            backend_pattern_config_dict.get(EXTRA_INPUTS_GETTER_DICT_KEY, None)  # type: ignore[arg-type]
+            backend_pattern_config_dict.get(EXTRA_INPUTS_GETTER_DICT_KEY)  # type: ignore[arg-type]
         )
         conf._set_num_tensor_args_to_observation_type(
             backend_pattern_config_dict.get(
