@@ -4,7 +4,14 @@ from contextlib import contextmanager
 from typing import TYPE_CHECKING
 
 import torch
-from torch.backends import __allow_nonbracketed_mutation, ContextProp, PropModule
+from torch.backends import (
+    __allow_nonbracketed_mutation,
+    _FP32Precision,
+    _get_fp32_precision_getter,
+    _set_fp32_precision_setter,
+    ContextProp,
+    PropModule,
+)
 
 
 def is_available():
@@ -12,6 +19,12 @@ def is_available():
     return torch._C._has_mkldnn
 
 
+def is_acl_available():
+    r"""Return whether PyTorch is built with MKL-DNN + ACL support."""
+    # pyrefly: ignore [missing-attribute]
+    return torch._C._has_mkldnn_acl
+
+
 VERBOSE_OFF = 0
 VERBOSE_ON = 1
 VERBOSE_ON_CREATION = 2
@@ -36,6 +49,7 @@ class verbose:
     .. code-block:: python
 
         import torch
+
         model(data)
         with torch.backends.mkldnn.verbose(torch.backends.mkldnn.VERBOSE_ON):
             model(data)
@@ -54,9 +68,9 @@ class verbose:
         if self.level == VERBOSE_OFF:
             return
         st = torch._C._verbose.mkldnn_set_verbose(self.level)
-        assert (
-            st
-        ), "Failed to set MKLDNN into verbose mode. Please consider to disable this verbose scope."
+        assert st, (
+            "Failed to set MKLDNN into verbose mode. Please consider to disable this verbose scope."
+        )
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
@@ -64,11 +78,14 @@ class verbose:
         return False
 
 
-def set_flags(_enabled=None, _deterministic=None, _allow_tf32=None):
+def set_flags(
+    _enabled=None, _deterministic=None, _allow_tf32=None, _fp32_precision="none"
+):
     orig_flags = (
         torch._C._get_mkldnn_enabled(),
         torch._C._get_mkldnn_deterministic(),
         torch._C._get_onednn_allow_tf32(),
+        torch._C._get_fp32_precision_getter("mkldnn", "all"),
     )
     if _enabled is not None:
         torch._C._set_mkldnn_enabled(_enabled)
@@ -76,13 +93,15 @@ def set_flags(_enabled=None, _deterministic=None, _allow_tf32=None):
         torch._C._set_mkldnn_deterministic(_deterministic)
     if _allow_tf32 is not None:
         torch._C._set_onednn_allow_tf32(_allow_tf32)
+    if _fp32_precision is not None:
+        torch._C._set_fp32_precision_setter("mkldnn", "all", _fp32_precision)
     return orig_flags
 
 
 @contextmanager
-def flags(enabled=False, deterministic=False, allow_tf32=True):
+def flags(enabled=False, deterministic=False, allow_tf32=True, fp32_precision="none"):
     with __allow_nonbracketed_mutation():
-        orig_flags = set_flags(enabled, deterministic, allow_tf32)
+        orig_flags = set_flags(enabled, deterministic, allow_tf32, fp32_precision)
     try:
         yield
     finally:
@@ -91,9 +110,6 @@ def flags(enabled=False, deterministic=False, allow_tf32=True):
 
 
 class MkldnnModule(PropModule):
-    def __init__(self, m, name):
-        super().__init__(m, name)
-
     def is_available(self):
         return is_available()
 
@@ -104,6 +120,13 @@ class MkldnnModule(PropModule):
     allow_tf32 = ContextProp(
         torch._C._get_onednn_allow_tf32, torch._C._set_onednn_allow_tf32
     )
+    matmul = _FP32Precision("mkldnn", "matmul")
+    conv = _FP32Precision("mkldnn", "conv")
+    rnn = _FP32Precision("mkldnn", "rnn")
+    fp32_precision = ContextProp(
+        _get_fp32_precision_getter("mkldnn", "all"),
+        _set_fp32_precision_setter("generic", "all"),
+    )
 
 
 if TYPE_CHECKING:
