@@ -16,11 +16,13 @@ import functools
 import inspect
 import io
 import logging
+import re
 import sys
 import textwrap
 import traceback
+from collections.abc import Iterator
 from dataclasses import dataclass, field
-from typing import Any, Dict, Iterator, List, Optional, Set, Tuple, TypeVar
+from typing import Any, Optional, TypeVar
 
 import torch
 import torch.cuda._gpu_trace as gpu_trace
@@ -41,6 +43,10 @@ SeqNum = int
 
 logger = logging.getLogger(__name__)
 
+# Note that this is only factories that take Tensor as input as they are
+# the ones we care about.
+FACTORY_FUNCTION_REGEX = re.compile("(new_.*|.*_like)")
+
 
 class AccessType(enum.Enum):
     READ = enum.auto()
@@ -69,7 +75,7 @@ class Access:
     seq_num: SeqNum
     stream: StreamId
     operator: str
-    aliases: List[str]
+    aliases: list[str]
     is_output: bool
     stack_trace: traceback.StackSummary
 
@@ -136,7 +142,7 @@ class UnsynchronizedAccessError(SynchronizationError):
 class CUDASanitizerErrors(Exception):
     """Wrapper class for errors reported by CUDA Sanitizer."""
 
-    def __init__(self, errors: List[SynchronizationError]):
+    def __init__(self, errors: list[SynchronizationError]):
         self.errors = errors
 
     def __str__(self):
@@ -156,13 +162,13 @@ class TensorInfo:
     """
 
     allocation_stack_trace: Optional[traceback.StackSummary]
-    reads: List[Access] = field(default_factory=list)
+    reads: list[Access] = field(default_factory=list)
     write: Optional[Access] = None
 
 
 class _TensorsAccessed:
     def __init__(self) -> None:
-        self.accesses: Dict[DataPtr, TensorInfo] = {}
+        self.accesses: dict[DataPtr, TensorInfo] = {}
 
     def ensure_tensor_exists(self, data_ptr: DataPtr) -> None:
         if data_ptr not in self.accesses:
@@ -194,7 +200,7 @@ class _TensorsAccessed:
         del self.accesses[data_ptr]
 
     def were_there_reads_since_last_write(self, data_ptr: DataPtr) -> bool:
-        return True if self.accesses[data_ptr].reads else False
+        return bool(self.accesses[data_ptr].reads)
 
     def get_allocation_stack_trace(
         self, data_ptr: DataPtr
@@ -204,7 +210,7 @@ class _TensorsAccessed:
     def get_write(self, data_ptr: DataPtr) -> Optional[Access]:
         return self.accesses[data_ptr].write
 
-    def get_reads(self, data_ptr: DataPtr) -> List[Access]:
+    def get_reads(self, data_ptr: DataPtr) -> list[Access]:
         return self.accesses[data_ptr].reads
 
     def add_read(self, data_ptr: DataPtr, access: Access) -> None:
@@ -217,9 +223,9 @@ class _TensorsAccessed:
 
 class StreamSynchronizations:
     def __init__(self) -> None:
-        self.current_sync_states: Dict[StreamId, Dict[StreamId, SeqNum]] = {}
-        self.recorded_sync_states: Dict[EventId, Dict[StreamId, SeqNum]] = {}
-        self.host_sync_state: Dict[StreamId, SeqNum] = {}
+        self.current_sync_states: dict[StreamId, dict[StreamId, SeqNum]] = {}
+        self.recorded_sync_states: dict[EventId, dict[StreamId, SeqNum]] = {}
+        self.host_sync_state: dict[StreamId, SeqNum] = {}
         self.create_stream(DEFAULT_STREAM_ID)
 
     def _ensure_stream_exists(self, stream: StreamId) -> None:
@@ -283,7 +289,7 @@ class StreamSynchronizations:
         self.recorded_sync_states[event] = self.current_sync_states[stream].copy()
 
     def _state_wait_for_other(
-        self, state: Dict[StreamId, SeqNum], other: Dict[StreamId, SeqNum]
+        self, state: dict[StreamId, SeqNum], other: dict[StreamId, SeqNum]
     ) -> None:
         for stream, seq_num in other.items():
             state[stream] = max(state.get(stream, -1), seq_num)
@@ -297,7 +303,7 @@ class StreamSynchronizations:
 
     def all_streams_wait_for_event(self, event: EventId) -> None:
         self._ensure_event_exists(event)
-        for stream in self.current_sync_states.keys():
+        for stream in self.current_sync_states:
             self.stream_wait_for_event(stream, event)
 
         self._state_wait_for_other(
@@ -344,12 +350,12 @@ class EventHandler:
     def _handle_kernel_launch(
         self,
         stream: StreamId,
-        read_only: Set[DataPtr],
-        read_write: Set[DataPtr],
-        outputs: Set[DataPtr],
+        read_only: set[DataPtr],
+        read_write: set[DataPtr],
+        outputs: set[DataPtr],
         operator: str,
-        tensor_aliases: Dict[int, List[str]],
-    ) -> List[SynchronizationError]:
+        tensor_aliases: dict[int, list[str]],
+    ) -> list[SynchronizationError]:
         def check_conflict(
             data_ptr: DataPtr, current_access: Access, previous_access: Optional[Access]
         ) -> None:
@@ -367,7 +373,7 @@ class EventHandler:
                     )
                 )
 
-        error_list: List[SynchronizationError] = []
+        error_list: list[SynchronizationError] = []
         self.seq_num += 1
         self.syncs.update_seq_num(stream, self.seq_num)
         stack_trace = traceback.StackSummary.extract(
@@ -457,15 +463,15 @@ class EventHandler:
         self.syncs.all_streams_wait_for_event(event)
 
 
-def zip_by_key(a: Dict[TK, TVa], b: Dict[TK, TVb]) -> Iterator[Tuple[TK, TVa, TVb]]:
+def zip_by_key(a: dict[TK, TVa], b: dict[TK, TVb]) -> Iterator[tuple[TK, TVa, TVb]]:
     for arg, value in a.items():
         if arg in b:
             yield arg, value, b[arg]
 
 
 def zip_arguments(
-    schema: torch.FunctionSchema, args: Tuple[Any, ...], kwargs: Dict[str, Any]
-) -> Iterator[Tuple[torch.Argument, Any]]:
+    schema: torch.FunctionSchema, args: tuple[Any, ...], kwargs: dict[str, Any]
+) -> Iterator[tuple[torch.Argument, Any]]:
     schema_args = schema.arguments[: len(args)]
     schema_kwargs = {arg.name: arg for arg in schema.arguments[len(args) :]}
 
@@ -477,15 +483,16 @@ def zip_arguments(
 
 class ArgumentHandler:
     def __init__(self) -> None:
-        self.dataptrs_read: Set[DataPtr] = set()
-        self.dataptrs_written: Set[DataPtr] = set()
-        self.tensor_aliases: Dict[DataPtr, List[str]] = {}
-        self.outputs: Set[DataPtr] = set()
+        self.dataptrs_read: set[DataPtr] = set()
+        self.dataptrs_written: set[DataPtr] = set()
+        self.tensor_aliases: dict[DataPtr, list[str]] = {}
+        self.outputs: set[DataPtr] = set()
 
     def _handle_argument(
         self,
         value: Any,
         is_write: bool,
+        metadata_only: bool,
         name: Optional[str] = None,
         is_output: bool = False,
     ) -> None:
@@ -493,7 +500,7 @@ class ArgumentHandler:
             data_ptr = value.data_ptr()
             if is_write:
                 self.dataptrs_written.add(data_ptr)
-            else:
+            elif not metadata_only:
                 self.dataptrs_read.add(data_ptr)
 
             self.tensor_aliases.setdefault(data_ptr, [])
@@ -505,23 +512,44 @@ class ArgumentHandler:
     def parse_inputs(
         self,
         schema: torch.FunctionSchema,
-        args: Tuple[Any, ...],
-        kwargs: Dict[str, Any],
+        args: tuple[Any, ...],
+        kwargs: dict[str, Any],
+        *,
+        is_factory: bool,
     ) -> None:
         for argument, value in zip_arguments(schema, args, kwargs):
             is_write = argument.alias_info is not None and argument.alias_info.is_write
+            # A change is metadata only if it is a view or a factory function that
+            # reads only metadata
+            metadata_only = is_factory or (
+                argument.alias_info is not None and not argument.alias_info.is_write
+            )
             pytree.tree_map_(
                 functools.partial(
-                    self._handle_argument, is_write=is_write, name=argument.name
+                    self._handle_argument,
+                    is_write=is_write,
+                    name=argument.name,
+                    metadata_only=metadata_only,
                 ),
                 value,
             )
 
-    def parse_outputs(self, outputs: Any) -> None:
-        pytree.tree_map_(
-            functools.partial(self._handle_argument, is_write=True, is_output=True),
-            outputs,
-        )
+    def parse_outputs(
+        self, schema: torch.FunctionSchema, outputs: Any, *, is_factory: bool
+    ) -> None:
+        for res, value in zip(schema.returns, (outputs,)):
+            metadata_only = is_factory or (
+                res.alias_info is not None and not res.alias_info.is_write
+            )
+            pytree.tree_map_(
+                functools.partial(
+                    self._handle_argument,
+                    is_write=not metadata_only,
+                    is_output=True,
+                    metadata_only=metadata_only,
+                ),
+                value,
+            )
 
 
 class CUDASanitizerDispatchMode(TorchDispatchMode):
@@ -563,12 +591,14 @@ class CUDASanitizerDispatchMode(TorchDispatchMode):
         if kwargs is None:
             kwargs = {}
 
+        is_factory = bool(FACTORY_FUNCTION_REGEX.match(func._schema.name))
+
         argument_handler = ArgumentHandler()
-        argument_handler.parse_inputs(func._schema, args, kwargs)
+        argument_handler.parse_inputs(func._schema, args, kwargs, is_factory=is_factory)
 
         outputs = func(*args, **kwargs)
 
-        argument_handler.parse_outputs(outputs)
+        argument_handler.parse_outputs(func._schema, outputs, is_factory=is_factory)
         errors = self.event_handler._handle_kernel_launch(
             torch.cuda.current_stream().cuda_stream,
             argument_handler.dataptrs_read - argument_handler.dataptrs_written,
@@ -602,9 +632,20 @@ class CUDASanitizer:
         self.dispatch.__enter__()
         self.enabled = True
 
+    def disable(self):
+        self.dispatch.__exit__(None, None, None)
+        self.enabled = False
+
     def __del__(self):
-        if self.enabled:
-            self.dispatch.__exit__(None, None, None)
+        # Since this object lifetime is linked to the `torch.cuda._sanitizer` python
+        # module, it often gets deleted as part of the overall `torch` module cleanup
+        # At that time, depending on CPython version, the torch.* module might be in
+        # different states of being already cleaned up.
+        # Similarly other imports might already have been cleaned up so `sys` might
+        # be already gone as well.
+        # Skip exiting the mode if it outlived the runtime.
+        if (sys is not None) and (not sys.is_finalizing()) and self.enabled:
+            self.disable()
 
 
 def enable_cuda_sanitizer():
