@@ -5,14 +5,14 @@ This is not intended to be imported directly; please use the exposed
 functionalities in `torch.jit`.
 """
 
-from typing import Any, List
+from typing import Any
 
 import torch
 from torch import TensorType
 from torch._C import Graph
 
 
-def apply_input_props_using_example(graph: Graph, example_input: List[Any]):
+def apply_input_props_using_example(graph: Graph, example_input: list[Any]) -> None:
     """
     Applies properties for each tensor in the graph inputs
     using the example supplied.
