@@ -115,6 +115,7 @@ node_start_tokens = {
     ast.Continue: "continue",
 }
 
+# pyrefly: ignore [no-matching-overload]
 pretty_node_names.update(
     {
         ast.AsyncFunctionDef: "async function definitions",
@@ -125,6 +126,7 @@ pretty_node_names.update(
     }
 )
 
+# pyrefly: ignore [no-matching-overload]
 node_start_tokens.update(
     {
         ast.AsyncFunctionDef: "async def",
@@ -135,6 +137,7 @@ node_start_tokens.update(
     }
 )
 
+# pyrefly: ignore [no-matching-overload]
 pretty_node_names.update(
     {
         ast.AnnAssign: "annotated assignments",
@@ -144,7 +147,7 @@ pretty_node_names.update(
 
 
 class FrontendError(Exception):
-    def __init__(self, source_range, msg):
+    def __init__(self, source_range, msg) -> None:
         self.source_range = source_range
         self.msg = msg
 
@@ -152,7 +155,7 @@ class FrontendError(Exception):
         # call stack when the FrontendError was raised
         self.error_report = torch._C.ErrorReport(self.source_range)
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.msg + self.error_report.what().lstrip()
 
 
@@ -161,7 +164,7 @@ class NotSupportedError(FrontendError):
 
 
 class UnsupportedNodeError(NotSupportedError):
-    def __init__(self, ctx, offending_node, reason=""):
+    def __init__(self, ctx, offending_node, reason="") -> None:
         # If we don't have a specific token, we default to length of 1
         node_type = type(offending_node)
         range_len = len(node_start_tokens.get(node_type, " "))
@@ -226,7 +229,7 @@ def get_class_properties(cls, self_name):
 def get_class_assigns(ctx, cls_ast):
     assigns = []
 
-    def maybe_build_assign(builder, entry):
+    def maybe_build_assign(builder, entry) -> None:
         nonlocal assigns
         try:
             assigns.append(builder(ctx, entry))
@@ -382,7 +385,7 @@ def get_jit_def(fn, def_name, self_name=None, is_classmethod=False):
 
 
 # TODO: more robust handling of recognizing ignore context manager
-def is_torch_jit_ignore_context_manager(stmt):
+def is_torch_jit_ignore_context_manager(stmt) -> bool:
     # checks if the statement is torch.jit.ignore context manager
     if isinstance(stmt.items[0].context_expr, ast.Call):
         # extract torch part
@@ -532,7 +535,7 @@ def build_ignore_context_manager(ctx, stmt):
                 outputs.append(OutputType(var_name, var_ann))
         return inputs, outputs
 
-    def create_unique_name_ext(ctx, stmt):
+    def create_unique_name_ext(ctx, stmt) -> str:
         # extension will be based on the full path filename plus
         # the line number of original context manager
         fn = re.sub(r"[^a-zA-Z0-9_]", "_", ctx.filename)
@@ -711,7 +714,7 @@ class StmtBuilder(Builder):
 
         # Disallow type annotations on instance attributes outside of __init__
         if (
-            type(stmt.target) == ast.Attribute
+            type(stmt.target) is ast.Attribute
             and stmt.target.value.id == "self"  # type: ignore[attr-defined]
             and ctx.funcname != "__init__"
         ):
@@ -859,6 +862,7 @@ class ExprBuilder(Builder):
         ast.RShift: ">>",
     }
 
+    # pyrefly: ignore [unsupported-operation]
     binop_map[ast.MatMult] = "@"
 
     unop_map = {
@@ -1220,6 +1224,7 @@ class ExprBuilder(Builder):
                 s += "{}"
                 args.append(build_expr(ctx, value.value))
             elif isinstance(value, ast.Constant):
+                # pyrefly: ignore [unsupported-operation]
                 s += value.value
             else:
                 raise NotSupportedError(r, "Unsupported value in JoinedStr")
