@@ -1,5 +1,3 @@
-from typing import Optional
-
 import torch.nn.functional as F
 from torch import Tensor
 from torch.nn.common_types import (
@@ -57,7 +55,7 @@ class _MaxPoolNd(Module):
     def __init__(
         self,
         kernel_size: _size_any_t,
-        stride: Optional[_size_any_t] = None,
+        stride: _size_any_t | None = None,
         padding: _size_any_t = 0,
         dilation: _size_any_t = 1,
         return_indices: bool = False,
@@ -298,7 +296,7 @@ class MaxPool3d(_MaxPoolNd):
 
     .. _link:
         https://github.com/vdumoulin/conv_arithmetic/blob/master/README.md
-    """  # noqa: E501
+    """
 
     kernel_size: _size_3_t
     stride: _size_3_t
@@ -389,7 +387,7 @@ class MaxUnpool1d(_MaxUnpoolNd):
     def __init__(
         self,
         kernel_size: _size_1_t,
-        stride: Optional[_size_1_t] = None,
+        stride: _size_1_t | None = None,
         padding: _size_1_t = 0,
     ) -> None:
         super().__init__()
@@ -398,7 +396,7 @@ class MaxUnpool1d(_MaxUnpoolNd):
         self.padding = _single(padding)
 
     def forward(
-        self, input: Tensor, indices: Tensor, output_size: Optional[list[int]] = None
+        self, input: Tensor, indices: Tensor, output_size: list[int] | None = None
     ) -> Tensor:
         """Runs the forward pass."""
         return F.max_unpool1d(
@@ -485,7 +483,7 @@ class MaxUnpool2d(_MaxUnpoolNd):
     def __init__(
         self,
         kernel_size: _size_2_t,
-        stride: Optional[_size_2_t] = None,
+        stride: _size_2_t | None = None,
         padding: _size_2_t = 0,
     ) -> None:
         super().__init__()
@@ -494,7 +492,7 @@ class MaxUnpool2d(_MaxUnpoolNd):
         self.padding = _pair(padding)
 
     def forward(
-        self, input: Tensor, indices: Tensor, output_size: Optional[list[int]] = None
+        self, input: Tensor, indices: Tensor, output_size: list[int] | None = None
     ) -> Tensor:
         """Runs the forward pass."""
         return F.max_unpool2d(
@@ -564,7 +562,7 @@ class MaxUnpool3d(_MaxUnpoolNd):
     def __init__(
         self,
         kernel_size: _size_3_t,
-        stride: Optional[_size_3_t] = None,
+        stride: _size_3_t | None = None,
         padding: _size_3_t = 0,
     ) -> None:
         super().__init__()
@@ -573,7 +571,7 @@ class MaxUnpool3d(_MaxUnpoolNd):
         self.padding = _triple(padding)
 
     def forward(
-        self, input: Tensor, indices: Tensor, output_size: Optional[list[int]] = None
+        self, input: Tensor, indices: Tensor, output_size: list[int] | None = None
     ) -> Tensor:
         """Runs the forward pass."""
         return F.max_unpool3d(
@@ -762,11 +760,11 @@ class AvgPool2d(_AvgPoolNd):
     def __init__(
         self,
         kernel_size: _size_2_t,
-        stride: Optional[_size_2_t] = None,
+        stride: _size_2_t | None = None,
         padding: _size_2_t = 0,
         ceil_mode: bool = False,
         count_include_pad: bool = True,
-        divisor_override: Optional[int] = None,
+        divisor_override: int | None = None,
     ) -> None:
         super().__init__()
         self.kernel_size = kernel_size
@@ -879,11 +877,11 @@ class AvgPool3d(_AvgPoolNd):
     def __init__(
         self,
         kernel_size: _size_3_t,
-        stride: Optional[_size_3_t] = None,
+        stride: _size_3_t | None = None,
         padding: _size_3_t = 0,
         ceil_mode: bool = False,
         count_include_pad: bool = True,
-        divisor_override: Optional[int] = None,
+        divisor_override: int | None = None,
     ) -> None:
         super().__init__()
         self.kernel_size = kernel_size
@@ -964,8 +962,8 @@ class FractionalMaxPool2d(Module):
     def __init__(
         self,
         kernel_size: _size_2_t,
-        output_size: Optional[_size_2_t] = None,
-        output_ratio: Optional[_ratio_2_t] = None,
+        output_size: _size_2_t | None = None,
+        output_ratio: _ratio_2_t | None = None,
         return_indices: bool = False,
         _random_samples=None,
     ) -> None:
@@ -1050,8 +1048,8 @@ class FractionalMaxPool3d(Module):
     def __init__(
         self,
         kernel_size: _size_3_t,
-        output_size: Optional[_size_3_t] = None,
-        output_ratio: Optional[_ratio_3_t] = None,
+        output_size: _size_3_t | None = None,
+        output_ratio: _ratio_3_t | None = None,
         return_indices: bool = False,
         _random_samples=None,
     ) -> None:
@@ -1106,7 +1104,7 @@ class _LPPoolNd(Module):
         self,
         norm_type: float,
         kernel_size: _size_any_t,
-        stride: Optional[_size_any_t] = None,
+        stride: _size_any_t | None = None,
         ceil_mode: bool = False,
     ) -> None:
         super().__init__()
@@ -1141,6 +1139,10 @@ class LPPool1d(_LPPoolNd):
         stride: a single int, the stride of the window. Default value is :attr:`kernel_size`
         ceil_mode: when True, will use `ceil` instead of `floor` to compute the output shape
 
+    Note:
+        When :attr:`ceil_mode` is ``True``, sliding windows may go off-bounds if they start within the
+        left padding or the input. Sliding windows that would start in the right padded region are ignored.
+
     Shape:
         - Input: :math:`(N, C, L_{in})` or :math:`(C, L_{in})`.
         - Output: :math:`(N, C, L_{out})` or :math:`(C, L_{out})`, where
@@ -1190,6 +1192,10 @@ class LPPool2d(_LPPoolNd):
         stride: the stride of the window. Default value is :attr:`kernel_size`
         ceil_mode: when True, will use `ceil` instead of `floor` to compute the output shape
 
+    Note:
+        When :attr:`ceil_mode` is ``True``, sliding windows may go off-bounds if they start within the
+        left padding or the input. Sliding windows that would start in the right padded region are ignored.
+
     Shape:
         - Input: :math:`(N, C, H_{in}, W_{in})` or :math:`(C, H_{in}, W_{in})`.
         - Output: :math:`(N, C, H_{out}, W_{out})` or :math:`(C, H_{out}, W_{out})`, where
@@ -1246,6 +1252,10 @@ class LPPool3d(_LPPoolNd):
         stride: the stride of the window. Default value is :attr:`kernel_size`
         ceil_mode: when True, will use `ceil` instead of `floor` to compute the output shape
 
+    Note:
+        When :attr:`ceil_mode` is ``True``, sliding windows may go off-bounds if they start within the
+        left padding or the input. Sliding windows that would start in the right padded region are ignored.
+
     Shape:
         - Input: :math:`(N, C, D_{in}, H_{in}, W_{in})` or :math:`(C, D_{in}, H_{in}, W_{in})`.
         - Output: :math:`(N, C, D_{out}, H_{out}, W_{out})` or
